@@ -1,16 +1,33 @@
 # -*- coding: utf-8 -*-
 
 """
-The ``mlflow.pyfunc`` module defines a generic :ref:`filesystem format <pyfunc-filesystem-format>`
-for Python models and provides utilities for saving to and loading from this format. The format is
-self contained in the sense that it includes all necessary information for anyone to load it and
-use it. Dependencies are either stored directly with the model or referenced via a Conda
-environment.
+The ``python_function`` model flavor serves as a default model interface for MLflow Python models.
+Any MLflow Python model is expected to be loadable as a ``python_function`` model.
+
+In addition, the ``mlflow.pyfunc`` module defines a generic :ref:`filesystem format
+<pyfunc-filesystem-format>` for Python models and provides utilities for saving to and loading from
+this format. The format is self contained in the sense that it includes all necessary information
+for anyone to load it and use it. Dependencies are either stored directly with the model or
+referenced via a Conda environment.
 
 The ``mlflow.pyfunc`` module also defines utilities for creating custom ``pyfunc`` models
 using frameworks and inference logic that may not be natively included in MLflow. See
 :ref:`pyfunc-create-custom`.
 
+.. _pyfunc-inference-api:
+
+*************
+Inference API
+*************
+
+Python function models are loaded as an instance of :py:class:`PyFuncModel
+<mlflow.pyfunc.PyFuncModel>`, which is an MLflow wrapper around the model implementation and model
+metadata (MLmodel file). You can score the model by calling the :py:func:`predict()
+<mlflow.pyfunc.PyFuncModel.predict>` method, which has the following signature::
+
+  predict(model_input: pandas.DataFrame) -> [numpy.ndarray | pandas.(Series | DataFrame)]
+
+
 .. _pyfunc-filesystem-format:
 
 *****************
@@ -42,10 +59,13 @@ following parameters:
          e.g. ``mlflow.sklearn``, it will be imported using ``importlib.import_module``.
          The imported module must contain a function with the following signature::
 
-          _load_pyfunc(path: string) -> <pyfunc model>
+          _load_pyfunc(path: string) -> <pyfunc model implementation>
 
          The path argument is specified by the ``data`` parameter and may refer to a file or
-         directory.
+         directory. The model implementation is expected to be an object with a
+         ``predict`` method with the following signature::
+
+           predict(model_input: pandas.DataFrame) -> [numpy.ndarray | pandas.(Series | DataFrame)]
 
 - code [optional]:
         Relative path to a directory containing the code packaged with this model.
@@ -92,19 +112,6 @@ following parameters:
     env: mlflow_env.yml
     main: sklearn_iris
 
-.. _pyfunc-inference-api:
-
-*************
-Inference API
-*************
-
-The convention for pyfunc models is to have a ``predict`` method or function with the following
-signature::
-
-    predict(model_input: pandas.DataFrame) -> [numpy.ndarray | pandas.Series | pandas.DataFrame]
-
-This convention is relied on by other MLflow components.
-
 .. _pyfunc-create-custom:
 
 ******************************
@@ -206,7 +213,7 @@ import mlflow
 import mlflow.pyfunc.model
 import mlflow.pyfunc.utils
 from mlflow.models import Model, ModelSignature, ModelInputExample
-
+from mlflow.models.model import MLMODEL_FILE_NAME
 from mlflow.models.utils import _save_example
 from mlflow.pyfunc.model import PythonModel, PythonModelContext, get_default_conda_env
 from mlflow.tracking.artifact_utils import _download_artifact_from_uri
@@ -449,7 +456,7 @@ def load_model(model_uri: str, suppress_warnings: bool = True) -> PyFuncModel:
                               messages will be emitted.
     """
     local_path = _download_artifact_from_uri(artifact_uri=model_uri)
-    model_meta = Model.load(os.path.join(local_path, "MLmodel"))
+    model_meta = Model.load(os.path.join(local_path, MLMODEL_FILE_NAME))
 
     conf = model_meta.flavors.get(FLAVOR_NAME)
     if conf is None:
@@ -983,7 +990,7 @@ def _save_model_with_loader_module_and_data_path(path, loader_module, data_path=
 
     mlflow.pyfunc.add_to_model(
         mlflow_model, loader_module=loader_module, code=code, data=data, env=conda_env_subpath)
-    mlflow_model.save(os.path.join(path, 'MLmodel'))
+    mlflow_model.save(os.path.join(path, MLMODEL_FILE_NAME))
     return mlflow_model
 
 
